"use strict";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var symbioteSymbols = require('symbiote-symbol');

var getActionCreatorDefault = function getActionCreatorDefault(type) {
  return function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return {
      type: type,
      payload: args
    };
  };
};

var defaultOptions = {
  namespace: undefined,
  defaultReducer: undefined,
  separator: '/'
  /**
   * @param {defaultOptions} options
   * @return {defaultOptions}
   */

};

var createOptions = function createOptions(options) {
  if (typeof options === 'string') {
    return Object.assign({}, defaultOptions, {
      namespace: options
    });
  }

  return Object.assign({}, defaultOptions, options);
};

var createSymbiote = function createSymbiote(initialState, actionsConfig) {
  var namespaceOptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  var handlersList = {};
  var options = createOptions(namespaceOptions);

  var traverseActions = function traverseActions(rootConfig) {
    var rootPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var actionsList = {};
    Object.keys(rootConfig).forEach(function (key) {
      var currentPath = rootPath.concat(key);
      var handler = rootConfig[key];
      var type = currentPath.join(options.separator);

      if (typeof handler === 'function') {
        var getActionCreator = handler[symbioteSymbols.getActionCreator] || getActionCreatorDefault;
        actionsList[key] = getActionCreator(type);

        actionsList[key].toString = function () {
          return type;
        };

        handlersList[type] = handler;
      } else {
        actionsList[key] = traverseActions(rootConfig[key], currentPath);
      }
    });
    return actionsList;
  };

  var actionsList = traverseActions(actionsConfig, options.namespace ? [options.namespace] : undefined);
  return {
    actions: actionsList,
    reducer: function reducer() {
      var previousState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments.length > 1 ? arguments[1] : undefined;
      var type = action.type,
          _action$payload = action.payload,
          args = _action$payload === void 0 ? [] : _action$payload;
      var handler = handlersList[type];

      if (handler) {
        return handler.apply(void 0, [previousState].concat(_toConsumableArray(args)));
      }

      return options.defaultReducer ? options.defaultReducer(previousState, action) : previousState;
    }
  };
};

module.exports = {
  createSymbiote: createSymbiote
};